@@ -1,5 +1,4 @@
 # -*- coding: utf8 -*-
-
 """ This module provides some classes to simplify acces to OpenERP server via xmlrpc.
     Some of these classes are may be not safe enough and should be used with carefully
 
@@ -48,19 +47,17 @@
 """
 
 
-import xmlrpclib
-from getpass import getpass
 import functools
 
 # project imports
-from utils import AttrDict
-from utils import ustr
+from openerp_proxy.connection import get_connector
+from openerp_proxy.exceptions import Error, ConnectorError
 
 
 __all__ = ('ERPProxyException', 'ERP_Proxy', 'ERP_Object', 'ERP_Record')
 
 
-class ERPProxyException(Exception):
+class ERPProxyException(Error):
     pass
 
 
@@ -69,11 +66,10 @@ class ERP_Proxy(object):
        A simple class to connect ot ERP via xml_rpc
        Should be initialized with following arguments:
 
-           >>> ERP_Proxy(dbname, host, user, pwd = getpass('Password: '), verbose = False)
+           >>> db = ERP_Proxy('dbname', 'host', 'user', pwd = 'Password', verbose = False)
 
        Allows access to ERP objects via dictionary syntax:
 
-           >>> db = ERP_Proxy(...)
            >>> db['sale.order']
                 ERP_Object: 'sale.order'
 
@@ -81,20 +77,20 @@ class ERP_Proxy(object):
 
     """
 
-    def __init__(self, dbname, host, user, pwd=None, port=8069, verbose=False):
+    def __init__(self, dbname, host, user, pwd, port=8069, protocol='xml-rpc', verbose=False):
         self.dbname = dbname
         self.host = host
         self.user = user
         self.port = port
-        self.pwd = pwd or getpass('Password: ')  # TODO: move getpass out from here
+        self.pwd = pwd
         self.verbose = verbose
+        self.protocol = protocol
+
+        self._connection = get_connector(protocol)(host, port, verbose=verbose)
 
         self.__objects = {}   # cached objects
-        self.__services = {}  # cached services
 
         # properties
-        self.__last_result = None
-        self.__last_result_wkf = None
         self.__registered_objects = None
 
         self.__use_execute_kw = None
@@ -110,32 +106,13 @@ class ERP_Proxy(object):
         """
         if self.__registered_objects is not None:
             return self.__registered_objects
-        t1, t2 = self.__last_result, self.__last_result_wkf
         ids = self.execute('ir.model', 'search', [])
         read = self.execute('ir.model', 'read', ids, ['model'])
         self.__registered_objects = [x['model'] for x in read]
-        self.__last_result, self.__last_result_wkf = t1, t2
         return self.__registered_objects
 
-    @property
-    def last_result(self):
-        """ Stores last value returned by 'execute' method
-        """
-        return self.__last_result
-
-    @property
-    def last_result_wkf(self):
-        """ Stores last value returned by 'execute_wkf' method
-        """
-        return self.__last_result_wkf
-
     def get_service(self, name):
-        if name in self.__services:
-            return self.__services[name]
-
-        service = xmlrpclib.ServerProxy('http://%s:%s/xmlrpc/%s' % (self.host, self.port, name), verbose=self.verbose)
-        self.__services[name] = service
-        return service
+        return self._connection.get_service(name)
 
     def connect(self):
         """ Connects to the server
@@ -156,9 +133,6 @@ class ERP_Proxy(object):
         """
         self.connect()
 
-        self.__last_result = None
-        self.__last_result_wkf = None
-
     # Report related methods
     def report(self, report_name, ids, context):
         """ Proxy to report service *report* method
@@ -191,7 +165,7 @@ class ERP_Proxy(object):
             try:
                 service.execute_kw(self.dbname, self.uid, self.pwd, 'ir.model', 'search', ([],), dict(limit=1))
                 self.__use_execute_kw = True
-            except xmlrpclib.Fault:
+            except ConnectorError:
                 self.__use_execute_kw = False
         return self.__use_execute_kw
 
@@ -202,17 +176,18 @@ class ERP_Proxy(object):
         service = self.get_service('object')
 
         if self.use_execute_kw():
-            self.__last_result = service.execute_kw(self.dbname, self.uid, self.pwd, obj, method, args, kwargs)
+            result = service.execute_kw(self.dbname, self.uid, self.pwd, obj, method, args, kwargs)
         else:
-            self.__last_result = service.execute(self.dbname, self.uid, self.pwd, obj, method, *args, **kwargs)
+            result = service.execute(self.dbname, self.uid, self.pwd, obj, method, *args, **kwargs)
 
-        return self.__last_result
+        return result
 
     def execute_wkf(self, object_name, signal, object_id):
         """ Triggers workflow event on specified object
         """
-        self.__last_result_wkf = self.get_service('object').exec_workflow(self.dbname, self.uid, self.pwd, object_name, signal, object_id)
-        return self.__last_result_wkf
+        service = self.get_service('object')
+        result_wkf = service.exec_workflow(self.dbname, self.uid, self.pwd, object_name, signal, object_id)
+        return result_wkf
 
     def get_obj(self, object_name):
         """ Returns wraper around openERP object 'object_name' which is instance of ERP_Object
@@ -242,13 +217,14 @@ class ERP_Proxy(object):
         return res
 
     def get_url(self):
-        return "%(user)s@%(host)s:%(port)s/%(database)s" % dict(user=self.user,
+        return "%(protocol)s://%(user)s@%(host)s:%(port)s/%(database)s" % dict(user=self.user,
                                                                 host=self.host,
                                                                 database=self.dbname,
-                                                                port=self.port)
+                                                                port=self.port,
+                                                                protocol=self.protocol)
 
     def __str__(self):
-        return "ERP_Proxy object: %s" % self.get_url()
+        return "ERP_Proxy: %s" % self.get_url()
     __repr__ = __str__
 
 
@@ -259,21 +235,11 @@ def MethodWrapper(erp_proxy, object_name, method_name):
         It is used in ERP_Object class.
     """
     def wrapper(*args, **kwargs):
-        try:
-            res = erp_proxy.execute(object_name, method_name, *args, **kwargs)
-        except xmlrpclib.Fault as exc:
-            msg = (u"A fault occured\n"
-                   u"Fault code: %s\n"
-                   u"Fault string: %s\n"
-                   u"" % (ustr(exc.faultCode),
-                          ustr(exc.faultString)))
-            msg = msg.encode('utf-8')
-            raise ERPProxyException(msg)
-        return res
+        return erp_proxy.execute(object_name, method_name, *args, **kwargs)
     return wrapper
 
 
-class ERP_Record(AttrDict):
+class ERP_Record(object):
     """ A simple class to wrap OpenERP records
 
         All fields could be used as dictionary items and as attributes of object
@@ -295,14 +261,13 @@ class ERP_Record(AttrDict):
         assert isinstance(obj, ERP_Object), "obj should be ERP_Object"
         assert isinstance(data, dict), "data should be dictionary structure returned by ERP_Object.read"
         self.__obj = obj
+        self.__data = data
         self.__related_objects = {}
-        self.__related_objects_o2m = {}
         self.__workflow_instance = None
-        self.update(data)
 
     def __dir__(self):
         res = dir(super(ERP_Record, self))
-        res.extend(self.keys())
+        res.extend(self.__data.keys())
         res.extend(['read', 'search', 'write', 'unlink', 'create'])
         return res
 
@@ -346,7 +311,7 @@ class ERP_Record(AttrDict):
         return []
 
     def __str__(self):
-        return "ERP_Record of %s,%s" % (self.__obj, self.id)
+        return "ERP_Record (%s, %s)" % (self.__obj.name, self.id)
     __repr__ = __str__
 
     def __getattribute__(self, name):
@@ -367,7 +332,8 @@ class ERP_Record(AttrDict):
         if name not in self.__related_objects or not cached:
             relation = self._get_columns_info()[name].relation
             rel_obj = self._get_proxy().get_obj(relation)
-            rel_id = self[name][0]   # Do not forged about relations in form [id, name]
+            rel_data = self[name]
+            rel_id = rel_data and rel_data[0] or False  # Do not forged about relations in form [id, name]
             self.__related_objects[name] = rel_obj.read_records(rel_id)
         return self.__related_objects[name]
 
@@ -375,46 +341,47 @@ class ERP_Record(AttrDict):
         """ Method used to fetch related objects by name of field that points to them
             using one2many relation
         """
-        if name not in self.__related_objects_o2m or not cached:
+        if name not in self.__related_objects or not cached:
             relation = self._get_columns_info()[name].relation
             rel_obj = self._get_proxy().get_obj(relation)
             rel_ids = self[name]   # Take in mind that field value is list of IDS
-            self.__related_objects_o2m[name] = rel_obj.read_records(rel_ids)
-        return self.__related_objects_o2m[name]
+            self.__related_objects[name] = rel_obj.read_records(rel_ids)
+        return self.__related_objects[name]
+
+    def __get_related_field(self, name):
+        """ Method to fetch related object's data
+        """
+        col_info = self._get_columns_info()[name]
+        if col_info.ttype == 'many2one':
+            return self.__get_many2one_rel_obj(name)
+        elif col_info.ttype == 'one2many' or col_info.ttype == 'many2many':
+            return self.__get_one2many_rel_obj(name)
+        else:
+            raise KeyError("There are no related field %s in model %s" % (name, self._get_obj().name))
 
     def __getitem__(self, name):
-        res = None
-        try:
-            res = super(ERP_Record, self).__getitem__(name)
-        except KeyError:
-            # Allow using '__obj' suffix in field name to retrive ERP_Record
-            # instance of object related via many2one or one2many or many2many
-            # This means next:
-            #    >>> o = db['sale.order.line'].read_records(1)
-            #    >>> o.order_id
-            #    ... [25, 'order_name']
-            #    >>> o.order_id__obj
-            #    ... ERP_Record of sale.order, 25
-            if name.endswith('__obj'):
-                fname = name[:-5]
-                col_info = self._get_columns_info()[fname]
-                if col_info.ttype == 'many2one':
-                    res = self.__get_many2one_rel_obj(fname)
-                elif col_info.ttype == 'one2many' or col_info.ttype == 'many2many':
-                    res = self.__get_one2many_rel_obj(fname)
-                else:
-                    raise
-            else:
-                raise
-
-        return res
+        # Allow using '__obj' suffix in field name to retrive ERP_Record
+        # instance of object related via many2one or one2many or many2many
+        # This means next:
+        #    >>> o = db['sale.order.line'].read_records(1)
+        #    >>> o.order_id
+        #    ... [25, 'order_name']
+        #    >>> o.order_id__obj
+        #    ... ERP_Record of sale.order, 25
+        if name.endswith('__obj'):
+            fname = name[:-5]
+            return self.__get_related_field(fname)
+        else:
+            return self.__data[name]
 
     def refresh(self):
-        self.update(self.__obj.read(self.id))
+        self.__data.update(self.__obj.read(self.id))
 
         # Update related objects cache
         self.__related_objects = {}
-        self.__related_objects_o2m = {}
+
+    def as_dict(self):
+        return self.__data.copy()
 
     def workflow_trg(self, signal):
         """ trigger's specified signal on record's related workflow
@@ -450,6 +417,10 @@ class ERP_Object(object):
         self.__columns_info = None
         self.__workflow = None
 
+    @property
+    def name(self):
+        return self.__obj_name
+
     def _get_proxy(self):
         return self.__erp_proxy
 
@@ -497,15 +468,20 @@ class ERP_Object(object):
                 >>> data = so_obj.search_records([('date','>=','2013-01-01')])
                 >>> for order in data:
                         order.write({'note': 'order date is %s'%order.date})
+
+            Additionally accepts keyword argument 'read_field' which can be used to specify
+            list of fields to read
         """
 
+        read_fields = kwargs.pop('read_fields', [])
+
         res = self.search(*args, **kwargs)
         if not res:
-            return False
-        if isinstance(res, (int, long)):
-            return ERP_Record(self, self.read(res))
-        if isinstance(res, (list, tuple)):
-            return [ERP_Record(self, data) for data in self.read(res)]
+            return []
+
+        if read_fields:
+            return self.read_records(res, read_fields)
+        return self.read_records(res)
 
     def read_records(self, ids, *args, **kwargs):
         """ Return instance or list of instances of ERP_Record class,
@@ -541,22 +517,3 @@ class ERP_Object(object):
     def __str__(self):
         return "ERP Object ('%s')" % self.__obj_name
     __repr__ = __str__
-
-
-def connect(dbname=None, host=None, user=None, pwd=None, port=8069, verbose=False):
-    """ Wraper aroun ERP_Proxy constructor class to simplify connect from shell.
-
-        @param dbname: name of database to connect to (will be asked interactvely if not provided)
-        @param host: host name to connect to (will be asked interactvely if not provided)
-        @param user: user name to connect as (will be asked interactvely if not provided)
-        @param pwd: password for selected user (will be asked interactvely if not provided)
-        @param port: port to connect to. (default: 8069)
-        @param verbose: to be verbose, or not to be. (default: False)
-        @return: ERP_Proxy object
-    """
-    host = host or raw_input('Server Host: ')
-    dbname = dbname or raw_input('Database name: ')
-    user = user or raw_input('ERP Login: ')
-    pwd = pwd or getpass('Password: ')
-    return ERP_Proxy(dbname=dbname, host=host, user=user, pwd=pwd, port=port, verbose=verbose)
-
