@@ -1,12 +1,16 @@
 import json
 import os.path
 import pprint
+from getpass import getpass
 
 # project imports
 from core import ERP_Proxy
 from plugin import ERP_PluginManager
 
 
+__all__ = ('ERP_Session',)
+
+
 class ERP_Session(object):
 
     """ Simple session manager which allows to manage databases easier
@@ -99,7 +103,7 @@ class ERP_Session(object):
         if isinstance(db, ERP_Proxy):
             return db
 
-        db = ERP_Proxy(**db)
+        db = ERP_Proxy(pwd=getpass('Password: '), **db)
         # injecting Plugins:
         db.plugins = ERP_PluginManager(db)
         # Plugins injected
@@ -112,7 +116,7 @@ class ERP_Session(object):
         """
         return self._databases.keys()
 
-    def connect(self, dbname=None, host=None, user=None, pwd=None, port=8069, verbose=False):
+    def connect(self, dbname=None, host=None, user=None, pwd=None, port=8069, protocol='xml-rpc', verbose=False):
         """ Wraper aroun ERP_Proxy constructor class to simplify connect from shell.
 
             @param dbname: name of database to connect to (will be asked interactvely if not provided)
@@ -126,16 +130,18 @@ class ERP_Session(object):
         host = host or raw_input('Server Host: ')
         dbname = dbname or raw_input('Database name: ')
         user = user or raw_input('ERP Login: ')
+        pwd = pwd or getpass("Password: ")
 
-        url = "%(user)s@%(host)s:%(port)s/%(database)s" % dict(user=user,
-                                                               host=host,
-                                                               database=dbname,
-                                                               port=port)
+        url = "%(protocol)s://%(user)s@%(host)s:%(port)s/%(database)s" % dict(user=user,
+                                                                              host=host,
+                                                                              database=dbname,
+                                                                              port=port,
+                                                                              protocol=protocol)
         db = self._databases.get(url, False)
         if isinstance(db, ERP_Proxy):
             return db
 
-        db = ERP_Proxy(dbname=dbname, host=host, user=user, pwd=pwd, port=port, verbose=verbose)
+        db = ERP_Proxy(dbname=dbname, host=host, user=user, pwd=pwd, port=port, protocol=protocol, verbose=verbose)
         self._add_db(url, db)
         return db
 
@@ -150,6 +156,7 @@ class ERP_Session(object):
                     'host': database.host,
                     'port': database.port,
                     'user': database.user,
+                    'protocol': database.protocol,
                     'verbose': database.verbose,
                 }
             else:
@@ -164,7 +171,7 @@ class ERP_Session(object):
         }
 
         with open(self.data_file, 'wt') as json_data:
-            json.dump(data, json_data)
+            json.dump(data, json_data, indent=4)
 
     def __getitem__(self, url_or_index):
         return self.get_db(url_or_index)
